/*
 *  LambertMaterial.cpp
 *  Assignment2
 *
 *  Created by Zach Gildersleeve on 1/28/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "LambertMaterial.h"

LambertMaterial::LambertMaterial (Color newColor, double newKdiffuse, double newKambient)
								: color(newColor), Kdiffuse(newKdiffuse), Kambient(newKambient) {
	
}

LambertMaterial::~LambertMaterial () {
	
}

void LambertMaterial::preprocess() {
	
}

void LambertMaterial::shade(Color &result, const RenderContext &context, const Ray &ray, const HitRecord &hit, int depth) const {
	
	// P = O + tV
	Point hitPosition = ray.getOrigin() + (ray.getDirection() * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	//costheta = N dot V
	double cosTheta = dot(normal, ray.getDirection());
	
	if (cosTheta > 0.0)   // should this be (cosTheta < 0) or (cosTheta > 0)
		normal = -normal;
		
	// color light = scene.ambient * Ka
	Scene *myScene = context.scene;
	Color light = myScene->getAmbient() * Kambient;
	
	const Object *world = myScene->object;
	
	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
	 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(normal, Lnormal);
		
		if (cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive())
				light += currentLightColor * Kdiffuse * cosPhi;
			
		}
	}
	
	result = light * color;
}