/*
 *  Scene.h
 *  Assignment2
 *
 *  Created by Zach Gildersleeve on 1/28/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef SCENE_H
#define SCENE_H

#include <vector>
#include "Background.h"
#include "Camera.h"
#include "Color.h"
#include "Image.h"
#include "Object.h"
#include "Light.h"
#include "RenderContext.h"
#include "Ray.h"
#include "Material.h"
#include "HitRecord.h"

class Background;
class Camera;
class Color;
class Image;
class Object;
class Light;

class Scene {

	public:
		Scene();
		virtual ~Scene();
		
		inline void setBackground(Background *newBackground) {
			background = newBackground;
		}
		
		inline void setCamera(Camera *cam) {
			camera = cam;
		}
		
		inline void setAmbient(Color amb) {
			ambient = amb;
		}
		
		inline void setImage(Image *img) {
			image = img;
		}
		
		inline void setObject(Object *obj) {
			object = obj;
		}
		
		inline void addLight(Light *lit) {
			lightList.push_back(lit);
		}
		
		inline Background* getBackground() const {
			return background;
		}
		
		inline Camera* getCamera() const {
			return camera;
		}
		
		inline Color getAmbient() const {
			return ambient;
		}
		
		inline Object* getObject() const {
			return object;
		}
		
		inline Image* getImage () const {
			return image;
		}
		
		const inline std::vector<Light*> &getLight() const {
			return lightList;
		}
		
		void preprocess();
		
		void render();
		void render(int x, int y);
		
		Scene(const Scene &s);
		Scene &operator=(const Scene &s);
	
	//private:
		Background *background;
		Camera *camera;
		Color ambient;
		Object *object;
		Image *image;
		std::vector<Light*> lightList;
		int xRes;
		int yRes;

};

#endif