/*
 *  Triangle.cpp
 *  Assignment3
 *
 *  Created by Zach Gildersleeve on 2/4/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "Triangle.h"

Triangle::Triangle(Material *newMaterial, const Point &newP1, const Point &newP2, const Point &newP3) 
					: Primitive(newMaterial), p1(newP1), p2(newP2), p3(newP3) {
	e1 = p1 - p3;
	e2 = p2 - p3;
	normalVector = cross(e1, e2);
	normalVector.normalize();
}

bool Triangle::intersect(HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	Vector r1 = cross(ray.getDirection(), e2);
	double denom = dot(e1, r1);
	if (fabs(denom) < 1e-6)
		return false;
	
	double invDenom = 1.0 / denom;
	
	Vector s = ray.getOrigin() - p3;
	
	double b1 = dot(s, r1) * invDenom;
	if (b1 < 0.0 || b1 > 1.0)
		return false;
	Vector r2 = cross(s, e1);
	double b2 = dot(ray.getDirection(), r2) * invDenom;
	
	if (b2 < 0.0 || b1 + b2 > 1.0)
		return false;
		
	double t = dot(e2, r2) * invDenom;
	
	hit.hit(t, this, material);
	return true;
	
}

void Triangle::normal(Vector &normal, const Point &hitPoint, const HitRecord &hit, const RenderContext &context, const Ray &ray) const {
	normal = normalVector;
}

