#include "Scene.h"
#include "ConstantBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"


static int assignment = 3;


Scene* make_scene()
{
  Scene* scene = new Scene();
  scene->setBackground(new ConstantBackground(Color(.3, .6, .5)));
  Group* world = new Group();
  world->addObject(new Sphere(new LambertMaterial(Color(1,1,1), .5, .5),
                              Point(25,25,120), 10.0));

  Material* box1matl = new LambertMaterial(Color(.1, .6, .8), .6, .4);
  world->addObject(new Box(box1matl,
                           Point(-4.5, -4.5, 0), Point(4.5, 4.5, 2)));
  Material* disc1matl = new LambertMaterial(Color(1, .2, .2), .6, .4);
  world->addObject(new Disc(disc1matl,
                            Point(3., -3.3, 2.5), Vector(-1, -1, 1), .5));
  Material* disc2matl = new LambertMaterial(Color(.2, 1, .2), .6, .4);
  world->addObject(new Disc(disc2matl,
                            Point(2.5, -3.3, 2.5), Vector(-1, -1, 1), .5));
  world->addObject(new Disc(disc1matl,
                            Point(2., -3.3, 2.5), Vector(-1, -1, 1), .5));
  world->addObject(new Disc(disc2matl,
                            Point(1.5, -3.3, 2.5), Vector(-1, -1, 1), .5));
  world->addObject(new Disc(disc1matl,
                            Point(1., -3.3, 2.5), Vector(-1, -1, 1), .5));
  Material* spherematl = new LambertMaterial(Color(1, 0, 0), .6, .4);
  world->addObject(new Sphere(spherematl,
                              Point(-2, 2, 5), .8));
  Material* ringmatl = new LambertMaterial(Color(.25, .25, 1), .6, .4);
  world->addObject(new Ring(ringmatl,
                            Point(-2, 2, 5), Vector(1, -1, 1), 1.2, 1.8));
  world->addObject(new Ring(ringmatl,
                            Point(-2, 2, 5), Vector(1, -1, 1), 2.2, 2.8));
  Material* trimatl = new LambertMaterial(Color(1, 1, 0), .6, .4);
  world->addObject(new Triangle(trimatl,
                                Point(1,1,4), Point(3,-.5,2), Point(3, 1.5, 6)));
  Material* box2matl = new LambertMaterial(Color(1, .4, .1), .6, .4);
  world->addObject(new Box(box2matl,
                           Point(-3.5, -3.5, 2), Point(0, 0, 2.35)));
  scene->setObject(world);

  scene->setAmbient(Color(.4, .4, .4));
  scene->setLight(new PointLight(Point(-40, -40, 100), Color(.9,.9,.9)));
  scene->setLight(new PointLight(Point(20, -40, 100), Color(.3,.1,.1)));

  scene->setCamera(new PinholeCamera(Point(-2.5, -20, 10),
                                     Point(0.5, 0, 2.5),
                                     Vector(0, 0, 1),
                                     26));

  return scene;
}

Scene* make_scene2()
{
	Scene* scene = new Scene();
	scene->setBackground(new ConstantBackground(Color(0.0, 0.0, 0.0)));
	Group* world = new Group();
	world->addObject(new Sphere(new LambertMaterial(Color(1,1,1), .5, .5),
								Point(25,25,120), 10.0));
	
	world->addObject(new Sphere(new LambertMaterial(Color(0.9, 0.9, 1.0), .3, .1),
								Point(15, 50, 10), 0.6));
	world->addObject(new Sphere(new LambertMaterial(Color(0.9, 0.5, 0.5), .3, .1),
								Point(15, 50, 15), 0.3));

	double scale = 2.0;
	Point saturnPoint = Point(-6.5, 50, 6);
	Vector ringVector = Vector(1.0, -0.35, 1.0);
	Material* spherematl = new LambertMaterial(Color(.553, .475, .043), .8, .5);
	world->addObject(new Sphere(spherematl,
								saturnPoint, 6.0 * scale));
	world->addObject(new Ring(new LambertMaterial(Color(0.2,0.2,0.2), .8, .8),
							  saturnPoint, ringVector, 6.8  * scale, 9.4  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(163.0/255.0,149.0/255.0,130.0/255.0), .3, .1),
							  saturnPoint, ringVector, 9.44  * scale, 10.0  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(163.0/255.0,149.0/255.0,130.0/255.0), .3, .1),
							  saturnPoint, ringVector, 10.1  * scale, 10.2  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(0.9,0.9,0.9), .8, .8),
							  saturnPoint, ringVector, 10.6  * scale, 10.8  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(103.0/255.0,107.0/255.0,119.0/255.0), .3, .1),
							  saturnPoint, ringVector, 11  * scale, 11.6  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(163.0/255.0,149.0/255.0,130.0/255.0), .6, .1),
							  saturnPoint, ringVector, 11.66  * scale, 12.2  * scale));
	world->addObject(new Ring(new LambertMaterial(Color(163.0/255.0,149.0/255.0,130.0/255.0), .2, .1),
							  saturnPoint, ringVector,12.3  * scale, 15.6  * scale));


	Material* disc1matl = new LambertMaterial(Color(0.8, 0.4, 0.2), 1.0, 1.0);
	world->addObject(new Disc(disc1matl,
							  Point(1.75, -31.1, -3.25), Vector(0, 1, 0), .5));							  
	world->addObject(new Disc(disc1matl,
							  Point(.65, -31.1, -3.25), Vector(0, 1, 0), .5));							  
	world->addObject(new Disc(disc1matl,
							  Point(2.85, -31.1, -3.25), Vector(0, 1, 0), .5));							  
					
	for (int i = 0; i < 50; i++)
	{	
		double r1 = (rand() % 10) * 0.1;
		double r2 = (rand() % 10) * 5;
		double r3 = (rand() % 10) * 5;
		Material* star = new LambertMaterial(Color(1.0, 1.0, 1.0), 0.6 + r1, 0.6 + r1);
		
		world->addObject(new Disc(star,
								  Point(-30 + r2, 60, -30 + r3), Vector(0, 1, 0), .1));				 
	}								  		  

	Material* shipMaterial = new LambertMaterial(Color(.6, .6, .6), .6, .5);
	Material* shipMaterial2 = new LambertMaterial(Color(.5, .5, .5), .6, .5);
	world->addObject(new Box(shipMaterial,
							 Point(0, -31, -4), Point(3.5, 10, -2.5)));
	world->addObject(new Box(shipMaterial,
							 Point(0.5, -30, -2.5), Point(3.0, 0, -1.5)));
	world->addObject(new Box(shipMaterial2,
							 Point(1, -30, -1.5), Point(1.25, -3, -1.3)));
	world->addObject(new Box(shipMaterial2,
							 Point(1.5, -30, -1.5), Point(1.75, -3, -1.3)));
	world->addObject(new Box(shipMaterial2,
							 Point(2.0, -30, -1.5), Point(2.25, -3, -1.3)));
	world->addObject(new Box(shipMaterial2,
							 Point(2.5, -30, -1.5), Point(2.75, -3, -1.3)));
							 
	world->addObject(new Triangle(shipMaterial2,
								  Point(0,-31,-3.1), Point(0,0,-3.3), Point(-3, -31, -3.5)));
	world->addObject(new Triangle(shipMaterial2,
								  Point(0,-31,-3.7), Point(0,0,-3.3), Point(-3, -31, -3.5)));
	world->addObject(new Triangle(shipMaterial2,
								  Point(0,-31,-3.7), Point(0,-31,-3.1), Point(-3, -31, -3.5)));
								  														 							 
	scene->setObject(world);
	
	scene->setAmbient(Color(.1, .1, .4));
	scene->setLight(new PointLight(Point(-40, -40, 100), Color(.9,.9,.9)));
	scene->setLight(new PointLight(Point(20, -40, 100), Color(.3,.1,.1)));
	scene->setLight(new PointLight(Point(50, -40, -100), Color(.2,.1,.1)));
	
	scene->setCamera(new PinholeCamera(Point(-4, -50, 0),
									   Point(0.0, 0.0, 0.0),
									   Vector(0, 0, 1),
									   30));
	
	return scene;
}

Scene* make_scene3()
{
	Scene* scene = new Scene();
	scene->setBackground(new ConstantBackground(Color(.3, .6, .5)));
	Group* world = new Group();
	world->addObject(new Sphere(new LambertMaterial(Color(1,1,1), .5, .5),
								Point(25,25,120), 10.0));
	
	Material* box1matl = new LambertMaterial(Color(.1, .6, .8), .6, .4);
	world->addObject(new Cylinder(box1matl,
							 Vector(0, 1, 0), Point(0, 0, 0), 1.0));
	scene->setObject(world);
	
	scene->setAmbient(Color(.4, .4, .4));
	scene->setLight(new PointLight(Point(-40, -40, 100), Color(.9,.9,.9)));
	scene->setLight(new PointLight(Point(20, -40, 100), Color(.3,.1,.1)));
	
	scene->setCamera(new PinholeCamera(Point(0, -2, 0),
									   Point(0, 0, 0),
									   Vector(0, 0, 1),
									   26));
	
	return scene;
}
